# -*- coding: utf-8 -*-

from . import Solar
from .util import SolarUtil


class SolarMonth:
    """
    阳历月
    """

    def __init__(self, year, month):
        self.__year = year
        self.__month = month

    @staticmethod
    def fromDate(date):
        return SolarMonth(date.year, date.month)

    @staticmethod
    def fromYm(year, month):
        return SolarMonth(year, month)

    def getYear(self):
        return self.__year

    def getMonth(self):
        return self.__month

    def toString(self):
        return "%d-%d" % (self.__year, self.__month)

    def toFullString(self):
        return "%d年%d月" % (self.__year, self.__month)

    def __str__(self):
        return self.toString()

    def getDays(self):
        """
        获取本月的阳历日期列表
        :return: 阳历日期列表
        """
        days = []
        d = Solar.fromYmd(self.__year, self.__month, 1)
        days.append(d)
        for i in range(1, SolarUtil.getDaysOfMonth(self.__year, self.__month)):
            days.append(d.next(i))
        return days

    def next(self, months):
        """
        获取往后推几个月的阳历月，如果要往前推，则月数用负数
        :param months: 月数
        :return: 阳历月
        """
        year = self.__year
        month = self.__month
        if months == 0:
            return SolarMonth.fromYm(year, month)
        n = abs(months)
        for i in range(1, n + 1):
            if months < 0:
                month -= 1
                if month < 1:
                    month = 12
                    year -= 1
            else:
                month += 1
                if month > 12:
                    month = 1
                    year += 1
        return SolarMonth.fromYm(year, month)



# -*- coding: utf-8 -*-


class TaoFestival:
    """
    道历节日
    """

    def __init__(self, name, remark=None):
        self.__name = name
        self.__remark = "" if remark is None else remark

    def getName(self):
        return self.__name

    def getRemark(self):
        return self.__remark

    def __str__(self):
        return self.toString()

    def toString(self):
        return self.__name

    def toFullString(self):
        s = self.__name
        if self.__remark is not None and len(self.__remark) > 0:
            s += "[" + self.__remark + "]"
        return s



# -*- coding: utf-8 -*-
from . import Solar, NineStar
from .util import ShouXingUtil, LunarUtil


class LunarYear:
    """
    农历年
    """

    YUAN = ("下", "上", "中")

    YUN = ("七", "八", "九", "一", "二", "三", "四", "五", "六")

    __LEAP_11 = (75, 94, 170, 238, 265, 322, 389, 469, 553, 583, 610, 678, 735, 754, 773, 849, 887, 936, 1050, 1069, 1126, 1145, 1164, 1183, 1259, 1278, 1308, 1373, 1403, 1441, 1460, 1498, 1555, 1593, 1612, 1631, 1642, 2033, 2128, 2147, 2242, 2614, 2728, 2910, 3062, 3244, 3339, 3616, 3711, 3730, 3825, 4007, 4159, 4197, 4322, 4341, 4379, 4417, 4531, 4599, 4694, 4713, 4789, 4808, 4971, 5085, 5104, 5161, 5180, 5199, 5294, 5305, 5476, 5677, 5696, 5772, 5791, 5848, 5886, 6049, 6068, 6144, 6163, 6258, 6402, 6440, 6497, 6516, 6630, 6641, 6660, 6679, 6736, 6774, 6850, 6869, 6899, 6918, 6994, 7013, 7032, 7051, 7070, 7089, 7108, 7127, 7146, 7222, 7271, 7290, 7309, 7366, 7385, 7404, 7442, 7461, 7480, 7491, 7499, 7594, 7624, 7643, 7662, 7681, 7719, 7738, 7814, 7863, 7882, 7901, 7939, 7958, 7977, 7996,
                 8034, 8053, 8072, 8091, 8121, 8159, 8186, 8216, 8235, 8254, 8273, 8311, 8330, 8341, 8349, 8368, 8444, 8463, 8474, 8493, 8531, 8569, 8588, 8626, 8664, 8683, 8694, 8702, 8713, 8721, 8751, 8789, 8808, 8816, 8827, 8846, 8884, 8903, 8922, 8941, 8971, 9036, 9066, 9085, 9104, 9123, 9142, 9161, 9180, 9199, 9218, 9256, 9294, 9313, 9324, 9343, 9362, 9381, 9419, 9438, 9476, 9514, 9533, 9544, 9552, 9563, 9571, 9582, 9601, 9639, 9658, 9666, 9677, 9696, 9734, 9753, 9772, 9791, 9802, 9821, 9886, 9897, 9916, 9935, 9954, 9973, 9992)

    __LEAP_12 = (37, 56, 113, 132, 151, 189, 208, 227, 246, 284, 303, 341, 360, 379, 417, 436, 458, 477, 496, 515, 534, 572, 591, 629, 648, 667, 697, 716, 792, 811, 830, 868, 906, 925, 944, 963, 982, 1001, 1020, 1039, 1058, 1088, 1153, 1202, 1221, 1240, 1297, 1335, 1392, 1411, 1422, 1430, 1517, 1525, 1536, 1574, 3358, 3472, 3806, 3988, 4751, 4941, 5066, 5123, 5275, 5343, 5438, 5457, 5495, 5533, 5552, 5715, 5810, 5829, 5905, 5924, 6421, 6535, 6793, 6812, 6888, 6907, 7002, 7184, 7260, 7279, 7374, 7556, 7746, 7757, 7776, 7833, 7852, 7871, 7966, 8015, 8110, 8129, 8148, 8224, 8243, 8338, 8406, 8425, 8482, 8501, 8520, 8558, 8596, 8607, 8615, 8645, 8740, 8778, 8835, 8865, 8930, 8960, 8979, 8998, 9017, 9055, 9074, 9093, 9112, 9150, 9188, 9237, 9275, 9332, 9351, 9370, 9408, 9427, 9446, 9457, 9465,
                 9495, 9560, 9590, 9628, 9647, 9685, 9715, 9742, 9780, 9810, 9818, 9829, 9848, 9867, 9905, 9924, 9943, 9962, 10000)

    __LEAP = {}

    __CACHE = {}

    for i in range(0, len(__LEAP_11)):
        __LEAP[__LEAP_11[i]] = 13
    for i in range(0, len(__LEAP_12)):
        __LEAP[__LEAP_12[i]] = 14

    def __init__(self, lunar_year):
        self.__year = lunar_year
        offset = lunar_year - 4
        year_gan_index = offset % 10
        year_zhi_index = offset % 12
        if year_gan_index < 0:
            year_gan_index += 10
        if year_zhi_index < 0:
            year_zhi_index += 12
        self.__ganIndex = year_gan_index
        self.__zhiIndex = year_zhi_index
        self.__months = []
        self.__jieQiJulianDays = []
        self.compute()

    @staticmethod
    def fromYear(lunar_year):
        if lunar_year not in LunarYear.__CACHE:
            obj = LunarYear(lunar_year)
            LunarYear.__CACHE[lunar_year] = obj
        else:
            obj = LunarYear.__CACHE[lunar_year]
        return obj

    def compute(self):
        from . import Lunar, Solar, LunarMonth
        # 节气(中午12点)，长度27
        jq = []
        # 合朔，即每月初一(中午12点)，长度16
        hs = []
        # 每月天数，长度15
        day_counts = []

        current_year = self.__year
        year = current_year - 2000

        # 从上年的大雪到下年的立春
        for i in range(0, len(Lunar.JIE_QI_IN_USE)):
            # 精确的节气
            t = 36525 * ShouXingUtil.saLonT((year + (17 + i) * 15.0 / 360) * ShouXingUtil.PI_2)
            t += ShouXingUtil.ONE_THIRD - ShouXingUtil.dtT(t)
            self.__jieQiJulianDays.append(t + Solar.J2000)
            # 按中午12点算的节气
            if 0 < i < 26:
                jq.append(round(t))
        # 冬至前的初一
        w = ShouXingUtil.calcShuo(jq[0])
        if w > jq[0]:
            w -= 29.5306
        # 递推每月初一
        for i in range(0, 16):
            hs.append(ShouXingUtil.calcShuo(w + 29.5306 * i))
        # 每月天数
        for i in range(0, 15):
            day_counts.append(int(hs[i + 1] - hs[i]))

        current_year_leap = -1
        if current_year in LunarYear.__LEAP:
            current_year_leap = LunarYear.__LEAP[current_year]
        else:
            if hs[13] <= jq[24]:
                i = 1
                while hs[i + 1] > jq[2 * i] and i < 13:
                    i += 1
                current_year_leap = i

        prev_year = current_year - 1
        prev_year_leap = -1
        if prev_year in LunarYear.__LEAP:
            prev_year_leap = LunarYear.__LEAP[prev_year] - 12

        y = prev_year
        m = 11
        for i in range(0, 15):
            cm = m
            is_next_leap = False
            if y == current_year and i == current_year_leap:
                cm = -m
            elif y == prev_year and i == prev_year_leap:
                cm = -m
            if y == current_year and i + 1 == current_year_leap:
                is_next_leap = True
            elif y == prev_year and i + 1 == prev_year_leap:
                is_next_leap = True
            self.__months.append(LunarMonth(y, cm, day_counts[i], hs[i] + Solar.J2000))
            if not is_next_leap:
                m += 1
            if m == 13:
                m = 1
                y += 1

    def getYear(self):
        return self.__year

    def getGanIndex(self):
        return self.__ganIndex

    def getZhiIndex(self):
        return self.__zhiIndex

    def getGan(self):
        return LunarUtil.GAN[self.__ganIndex + 1]

    def getZhi(self):
        return LunarUtil.ZHI[self.__zhiIndex + 1]

    def getGanZhi(self):
        return self.getGan() + self.getZhi()

    def toString(self):
        return str(self.__year) + ""

    def toFullString(self):
        return "%d年" % self.__year

    def __str__(self):
        return self.toString()

    def getMonths(self):
        return self.__months

    def getJieQiJulianDays(self):
        return self.__jieQiJulianDays

    def getLeapMonth(self):
        """
        获取闰月
        :return: 闰月数字，1代表闰1月，0代表无闰月
        """
        for i in range(0, len(self.__months)):
            m = self.__months[i]
            if m.getYear() == self.__year and m.isLeap():
                return abs(m.getMonth())
        return 0

    def getMonth(self, lunar_month):
        """
        获取农历月
        :param lunar_month: 闰月数字，1代表闰1月，0代表无闰月
        :return: 农历月
        """
        for i in range(0, len(self.__months)):
            m = self.__months[i]
            if m.getYear() == self.__year and m.getMonth() == lunar_month:
                return m
        return None

    def __getZaoByGan(self, index, name):
        offset = index - Solar.fromJulianDay(self.getMonth(1).getFirstJulianDay()).getLunar().getDayGanIndex()
        if offset < 0:
            offset += 10
        return name.replace("几", LunarUtil.NUMBER[offset + 1], 1)

    def __getZaoByZhi(self, index, name):
        offset = index - Solar.fromJulianDay(self.getMonth(1).getFirstJulianDay()).getLunar().getDayZhiIndex()
        if offset < 0:
            offset += 12
        return name.replace("几", LunarUtil.NUMBER[offset + 1], 1)

    def getTouLiang(self):
        return self.__getZaoByZhi(0, "几鼠偷粮")

    def getCaoZi(self):
        return self.__getZaoByZhi(0, "草子几分")

    def getGengTian(self):
        """
        获取耕田（正月第一个丑日是初几，就是几牛耕田）
        :return: 耕田，如：六牛耕田
        """
        return self.__getZaoByZhi(1, "几牛耕田")

    def getHuaShou(self):
        return self.__getZaoByZhi(3, "花收几分")

    def getZhiShui(self):
        """
        获取治水（正月第一个辰日是初几，就是几龙治水）
        :return: 治水，如：二龙治水
        """
        return self.__getZaoByZhi(4, "几龙治水")

    def getTuoGu(self):
        return self.__getZaoByZhi(6, "几马驮谷")

    def getQiangMi(self):
        return self.__getZaoByZhi(9, "几鸡抢米")

    def getKanCan(self):
        return self.__getZaoByZhi(9, "几姑看蚕")

    def getGongZhu(self):
        return self.__getZaoByZhi(11, "几屠共猪")

    def getJiaTian(self):
        return self.__getZaoByGan(0, "甲田几分")

    def getFenBing(self):
        """
        获取分饼（正月第一个丙日是初几，就是几人分饼）
        :return: 分饼，如：六人分饼
        """
        return self.__getZaoByGan(2, "几人分饼")

    def getDeJin(self):
        """
        获取得金（正月第一个辛日是初几，就是几日得金）
        :return: 得金，如：一日得金
        """
        return self.__getZaoByGan(7, "几日得金")

    def getRenBing(self):
        return self.__getZaoByGan(2, self.__getZaoByZhi(2, "几人几丙"))

    def getRenChu(self):
        return self.__getZaoByGan(3, self.__getZaoByZhi(2, "几人几锄"))

    def getYuan(self):
        return LunarYear.YUAN[int((self.__year + 2696) / 60) % 3] + "元"

    def getYun(self):
        return LunarYear.YUN[int((self.__year + 2696) / 20) % 9] + "运"

    def getNineStar(self):
        index = LunarUtil.getJiaZiIndex(self.getGanZhi()) + 1
        yuan = int((self.__year + 2696) / 60) % 3
        offset = (62 + yuan * 3 - index) % 9
        if 0 == offset:
            offset = 9
        return NineStar.fromIndex(offset - 1)

    def getPositionXi(self):
        return LunarUtil.POSITION_XI[self.__ganIndex + 1]

    def getPositionXiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionXi()]

    def getPositionYangGui(self):
        return LunarUtil.POSITION_YANG_GUI[self.__ganIndex + 1]

    def getPositionYangGuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionYangGui()]

    def getPositionYinGui(self):
        return LunarUtil.POSITION_YIN_GUI[self.__ganIndex + 1]

    def getPositionYinGuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionYinGui()]

    def getPositionFu(self, sect=2):
        return (LunarUtil.POSITION_FU if 1 == sect else LunarUtil.POSITION_FU_2)[self.__ganIndex + 1]

    def getPositionFuDesc(self, sect=2):
        return LunarUtil.POSITION_DESC[self.getPositionFu(sect)]

    def getPositionCai(self):
        return LunarUtil.POSITION_CAI[self.__ganIndex + 1]

    def getPositionCaiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionCai()]

    def getPositionTaiSui(self):
        return LunarUtil.POSITION_TAI_SUI_YEAR[self.__zhiIndex]

    def getPositionTaiSuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionTaiSui()]

    def next(self, n):
        """
        获取往后推几年的阴历年，如果要往前推，则年数用负数
        :param n: 年数
        :return: 阴历年
        """
        return LunarYear.fromYear(self.__year + n)



# -*- coding: utf-8 -*-
from . import NineStar
from .util import LunarUtil


class LunarTime:
    """
    时辰
    """

    def __init__(self, lunar_year, lunar_month, lunar_day, hour, minute, second):
        from . import Lunar
        self.__lunar = Lunar.fromYmdHms(lunar_year, lunar_month, lunar_day, hour, minute, second)
        self.__zhiIndex = LunarUtil.getTimeZhiIndex("%02d:%02d" % (hour, minute))
        self.__ganIndex = (self.__lunar.getDayGanIndexExact() % 5 * 2 + self.__zhiIndex) % 10

    @staticmethod
    def fromYmdHms(lunar_year, lunar_month, lunar_day, hour, minute, second):
        return LunarTime(lunar_year, lunar_month, lunar_day, hour, minute, second)

    def getGan(self):
        return LunarUtil.GAN[self.__ganIndex + 1]

    def getZhi(self):
        return LunarUtil.ZHI[self.__zhiIndex + 1]

    def getGanZhi(self):
        return self.getGan() + self.getZhi()

    def getShengXiao(self):
        return LunarUtil.SHENGXIAO[self.__zhiIndex + 1]

    def getPositionXi(self):
        return LunarUtil.POSITION_XI[self.__ganIndex + 1]

    def getPositionXiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionXi()]

    def getPositionYangGui(self):
        return LunarUtil.POSITION_YANG_GUI[self.__ganIndex + 1]

    def getPositionYangGuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionYangGui()]

    def getPositionYinGui(self):
        return LunarUtil.POSITION_YIN_GUI[self.__ganIndex + 1]

    def getPositionYinGuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionYinGui()]

    def getPositionFu(self, sect=2):
        return (LunarUtil.POSITION_FU if 1 == sect else LunarUtil.POSITION_FU_2)[self.__ganIndex + 1]

    def getPositionFuDesc(self, sect=2):
        return LunarUtil.POSITION_DESC[self.getPositionFu(sect)]

    def getPositionCai(self):
        return LunarUtil.POSITION_CAI[self.__ganIndex + 1]

    def getPositionCaiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionCai()]

    def getChong(self):
        return LunarUtil.CHONG[self.__zhiIndex + 1]

    def getChongGan(self):
        return LunarUtil.CHONG_GAN[self.__ganIndex + 1]

    def getChongGanTie(self):
        return LunarUtil.CHONG_GAN_TIE[self.getGan()]

    def getChongShengXiao(self):
        chong = self.getChong()
        for i in range(0, len(LunarUtil.ZHI)):
            if LunarUtil.ZHI[i] == chong:
                return LunarUtil.SHENGXIAO[i]
        return ""

    def getChongDesc(self):
        return "(" + self.getChongGan() + self.getChong() + ")" + self.getChongShengXiao()

    def getSha(self):
        return LunarUtil.SHA[self.getZhi()]

    def getNaYin(self):
        return LunarUtil.NAYIN[self.getGanZhi()]

    def getTianShen(self):
        offset = LunarUtil.ZHI_TIAN_SHEN_OFFSET[self.__lunar.getDayZhiExact()]
        return LunarUtil.TIAN_SHEN[(self.__zhiIndex + offset) % 12 + 1]

    def getTianShenType(self):
        return LunarUtil.TIAN_SHEN_TYPE[self.getTianShen()]

    def getTianShenLuck(self):
        return LunarUtil.TIAN_SHEN_TYPE_LUCK[self.getTianShenType()]

    def getYi(self):
        """
        获取时宜
        :return: 宜
        """
        return LunarUtil.getTimeYi(self.__lunar.getDayInGanZhiExact(), self.getGanZhi())

    def getJi(self):
        """
        获取时忌
        :return: 忌
        """
        return LunarUtil.getTimeJi(self.__lunar.getDayInGanZhiExact(), self.getGanZhi())

    def getNineStar(self):
        solar_ymd = self.__lunar.getSolar().toYmd()
        jie_qi = self.__lunar.getJieQiTable()
        asc = False
        if jie_qi["冬至"] <= solar_ymd < jie_qi["夏至"]:
            asc = True
        start = 7 if asc else 3
        day_zhi = self.__lunar.getDayZhi()
        if day_zhi in "子午卯酉":
            start = 1 if asc else 9
        elif day_zhi in "辰戌丑未":
            start = 4 if asc else 6
        index = start + self.__zhiIndex - 1 if asc else start - self.__zhiIndex - 1

        if index > 8:
            index -= 9
        if index < 0:
            index += 9
        return NineStar.fromIndex(index)

    def getGanIndex(self):
        return self.__ganIndex

    def getZhiIndex(self):
        return self.__zhiIndex

    def __str__(self):
        return self.toString()

    def toString(self):
        return self.getGanZhi()

    def getXun(self):
        """
        获取时辰所在旬
        :return: 旬
        """
        return LunarUtil.getXun(self.getGanZhi())

    def getXunKong(self):
        """
        获取值时空亡
        :return: 空亡(旬空)
        """
        return LunarUtil.getXunKong(self.getGanZhi())

    def getMinHm(self):
        hour = self.__lunar.getHour()
        if hour < 1:
            return "00:00"
        elif hour > 22:
            return "23:00"
        return "%02d:00", hour - 1 if hour % 2 == 0 else hour

    def getMaxHm(self):
        hour = self.__lunar.getHour()
        if hour < 1:
            return "00:59"
        elif hour > 22:
            return "23:59"
        return "%02d:59", hour + 1 if hour % 2 != 0 else hour



# -*- coding: utf-8 -*-


class Fu:
    """
    三伏
    <p>从夏至后第3个庚日算起，初伏为10天，中伏为10天或20天，末伏为10天。当夏至与立秋之间出现4个庚日时中伏为10天，出现5个庚日则为20天。</p>
    """

    def __init__(self, name, index):
        self.__name = name
        self.__index = index

    def getName(self):
        return self.__name

    def setName(self, name):
        self.__name = name

    def getIndex(self):
        return self.__index

    def setIndex(self, index):
        self.__index = index

    def __str__(self):
        return self.toString()

    def toString(self):
        return self.__name

    def toFullString(self):
        return "%s第%d天" % (self.__name, self.__index)



# -*- coding: utf-8 -*-

from . import SolarMonth


class SolarYear:
    """
    阳历年
    """

    MONTH_COUNT = 12

    def __init__(self, year):
        self.__year = year

    @staticmethod
    def fromDate(date):
        return SolarYear(date.year)

    @staticmethod
    def fromYear(year):
        return SolarYear(year)

    def getYear(self):
        return self.__year

    def toString(self):
        return str(self.__year)

    def toFullString(self):
        return "%d年" % self.__year

    def __str__(self):
        return self.toString()

    def getMonths(self):
        """
        获取本年的阳历月列表
        :return: 阳历月列表
        """
        months = []
        m = SolarMonth.fromYm(self.__year, 1)
        months.append(m)
        for i in range(1, SolarYear.MONTH_COUNT):
            months.append(m.next(i))
        return months

    def next(self, years):
        """
        获取往后推几年的阳历年，如果要往前推，则月数用负数
        :param years: 年数
        :return: 阳历年
        """
        return SolarYear.fromYear(self.__year + years)



# -*- coding: utf-8 -*-


class FotoFestival:
    """
    佛历因果犯忌
    """

    def __init__(self, name, result=None, every_month=False, remark=None):
        self.__name = name
        self.__result = "" if result is None else result
        self.__everyMonth = every_month
        self.__remark = "" if remark is None else remark

    def getName(self):
        return self.__name

    def getResult(self):
        return self.__result

    def isEveryMonth(self):
        return self.__everyMonth

    def getRemark(self):
        return self.__remark

    def __str__(self):
        return self.toString()

    def toString(self):
        return self.__name

    def toFullString(self):
        s = self.__name
        if self.__result is not None and len(self.__result) > 0:
            s += " " + self.__result
        if self.__remark is not None and len(self.__remark) > 0:
            s += " " + self.__remark
        return s



# -*- coding: utf-8 -*-
from .util import LunarUtil


class EightChar:
    """
    八字
    """

    MONTH_ZHI = ("", "寅", "卯", "辰", "巳", "午", "未", "申", "酉", "戌", "亥", "子", "丑")

    CHANG_SHENG = ("长生", "沐浴", "冠带", "临官", "帝旺", "衰", "病", "死", "墓", "绝", "胎", "养")

    __CHANG_SHENG_OFFSET = {
        "甲": 1,
        "丙": 10,
        "戊": 10,
        "庚": 7,
        "壬": 4,
        "乙": 6,
        "丁": 9,
        "己": 9,
        "辛": 0,
        "癸": 3
    }

    def __init__(self, lunar):
        self.__sect = 2
        self.__lunar = lunar

    @staticmethod
    def fromLunar(lunar):
        return EightChar(lunar)

    def toString(self):
        return self.getYear() + " " + self.getMonth() + " " + self.getDay() + " " + self.getTime()

    def __str__(self):
        return self.toString()

    def getSect(self):
        return self.__sect

    def setSect(self, sect):
        self.__sect = sect

    def getYear(self):
        """
        获取年柱
        :return: 年柱
        """
        return self.__lunar.getYearInGanZhiExact()

    def getYearGan(self):
        """
        获取年干
        :return: 天干
        """
        return self.__lunar.getYearGanExact()

    def getYearZhi(self):
        """
        获取年支
        :return: 地支
        """
        return self.__lunar.getYearZhiExact()

    def getYearHideGan(self):
        """
        获取年柱地支藏干，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 天干
        """
        return LunarUtil.ZHI_HIDE_GAN.get(self.getYearZhi())

    def getYearWuXing(self):
        """
        获取年柱五行
        :return: 五行
        """
        return LunarUtil.WU_XING_GAN.get(self.getYearGan()) + LunarUtil.WU_XING_ZHI.get(self.getYearZhi())

    def getYearNaYin(self):
        """
        获取年柱纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getYear())

    def getYearShiShenGan(self):
        """
        获取年柱天干十神
        :return: 十神
        """
        return LunarUtil.SHI_SHEN_GAN.get(self.getDayGan() + self.getYearGan())

    def __getShiShenZhi(self, zhi):
        hide_gan = LunarUtil.ZHI_HIDE_GAN.get(zhi)
        arr = []
        for gan in hide_gan:
            arr.append(LunarUtil.SHI_SHEN_ZHI.get(self.getDayGan() + zhi + gan))
        return arr

    def getYearShiShenZhi(self):
        """
        获取年柱地支十神，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 十神
        """
        return self.__getShiShenZhi(self.getYearZhi())

    def getDayGanIndex(self):
        return self.__lunar.getDayGanIndexExact2() if 2 == self.__sect else self.__lunar.getDayGanIndexExact()

    def getDayZhiIndex(self):
        return self.__lunar.getDayZhiIndexExact2() if 2 == self.__sect else self.__lunar.getDayZhiIndexExact()

    def __getDiShi(self, zhi_index):
        offset = self.__CHANG_SHENG_OFFSET.get(self.getDayGan())
        index = offset + (zhi_index if self.getDayGanIndex() % 2 == 0 else -zhi_index)
        if index >= 12:
            index -= 12
        if index < 0:
            index += 12
        return EightChar.CHANG_SHENG[index]

    def getYearDiShi(self):
        """
        获取年柱地势（长生十二神）
        :return: 地势
        """
        return self.__getDiShi(self.__lunar.getYearZhiIndexExact())

    def getMonth(self):
        """
        获取月柱
        :return: 月柱
        """
        return self.__lunar.getMonthInGanZhiExact()

    def getMonthGan(self):
        """
        获取月干
        :return: 天干
        """
        return self.__lunar.getMonthGanExact()

    def getMonthZhi(self):
        """
        获取月支
        :return: 地支
        """
        return self.__lunar.getMonthZhiExact()

    def getMonthHideGan(self):
        """
        获取月柱地支藏干，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 天干
        """
        return LunarUtil.ZHI_HIDE_GAN.get(self.getMonthZhi())

    def getMonthWuXing(self):
        """
        获取月柱五行
        :return: 五行
        """
        return LunarUtil.WU_XING_GAN.get(self.getMonthGan()) + LunarUtil.WU_XING_ZHI.get(self.getMonthZhi())

    def getMonthNaYin(self):
        """
        获取月柱纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getMonth())

    def getMonthShiShenGan(self):
        """
        获取月柱天干十神
        :return: 十神
        """
        return LunarUtil.SHI_SHEN_GAN.get(self.getDayGan() + self.getMonthGan())

    def getMonthShiShenZhi(self):
        """
        获取月柱地支十神，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 十神
        """
        return self.__getShiShenZhi(self.getMonthZhi())

    def getMonthDiShi(self):
        """
        获取月柱地势（长生十二神）
        :return: 地势
        """
        return self.__getDiShi(self.__lunar.getMonthZhiIndexExact())

    def getDay(self):
        """
        获取日柱
        :return: 日柱
        """
        return self.__lunar.getDayInGanZhiExact2() if 2 == self.__sect else self.__lunar.getDayInGanZhiExact()

    def getDayGan(self):
        """
        获取日干
        :return: 天干
        """
        return self.__lunar.getDayGanExact2() if 2 == self.__sect else self.__lunar.getDayGanExact()

    def getDayZhi(self):
        """
        获取日支
        :return: 地支
        """
        return self.__lunar.getDayZhiExact2() if 2 == self.__sect else self.__lunar.getDayZhiExact()

    def getDayHideGan(self):
        """
        获取日柱地支藏干，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 天干
        """
        return LunarUtil.ZHI_HIDE_GAN.get(self.getDayZhi())

    def getDayWuXing(self):
        """
        获取日柱五行
        :return: 五行
        """
        return LunarUtil.WU_XING_GAN.get(self.getDayGan()) + LunarUtil.WU_XING_ZHI.get(self.getDayZhi())

    def getDayNaYin(self):
        """
        获取日柱纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getDay())

    def getDayShiShenGan(self):
        """
        获取日柱天干十神，也称日元、日干
        :return: 十神
        """
        return "日主"

    def getDayShiShenZhi(self):
        """
        获取日柱地支十神，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 十神
        """
        return self.__getShiShenZhi(self.getDayZhi())

    def getDayDiShi(self):
        """
        获取日柱地势（长生十二神）
        :return: 地势
        """
        return self.__getDiShi(self.getDayZhiIndex())

    def getTime(self):
        """
        获取时柱
        :return: 时柱
        """
        return self.__lunar.getTimeInGanZhi()

    def getTimeGan(self):
        """
        获取时干
        :return: 天干
        """
        return self.__lunar.getTimeGan()

    def getTimeZhi(self):
        """
        获取时支
        :return: 地支
        """
        return self.__lunar.getTimeZhi()

    def getTimeHideGan(self):
        """
        获取时柱地支藏干，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 天干
        """
        return LunarUtil.ZHI_HIDE_GAN.get(self.getTimeZhi())

    def getTimeWuXing(self):
        """
        获取时柱五行
        :return: 五行
        """
        return LunarUtil.WU_XING_GAN.get(self.getTimeGan()) + LunarUtil.WU_XING_ZHI.get(self.getTimeZhi())

    def getTimeNaYin(self):
        """
        获取时柱纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getTime())

    def getTimeShiShenGan(self):
        """
        获取时柱天干十神
        :return: 十神
        """
        return LunarUtil.SHI_SHEN_GAN.get(self.getDayGan() + self.getTimeGan())

    def getTimeShiShenZhi(self):
        """
        获取时柱地支十神，由于藏干分主气、余气、杂气，所以返回结果可能为1到3个元素
        :return: 十神
        """
        return self.__getShiShenZhi(self.getTimeZhi())

    def getTimeDiShi(self):
        """
        获取时柱地势（长生十二神）
        :return: 地势
        """
        return self.__getDiShi(self.__lunar.getTimeZhiIndex())

    def getTaiYuan(self):
        """
        获取胎元
        :return: 胎元
        """
        gan_index = self.__lunar.getMonthGanIndexExact() + 1
        if gan_index >= 10:
            gan_index -= 10
        zhi_index = self.__lunar.getMonthZhiIndexExact() + 3
        if zhi_index >= 12:
            zhi_index -= 12
        return LunarUtil.GAN[gan_index + 1] + LunarUtil.ZHI[zhi_index + 1]

    def getTaiYuanNaYin(self):
        """
        获取胎元纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getTaiYuan())

    def getMingGong(self):
        """
        获取命宫
        :return: 命宫
        """
        month_zhi_index = 0
        time_zhi_index = 0
        for i in range(0, len(EightChar.MONTH_ZHI)):
            zhi = EightChar.MONTH_ZHI[i]
            if self.__lunar.getMonthZhiExact() == zhi:
                month_zhi_index = i

            if self.__lunar.getTimeZhi() == zhi:
                time_zhi_index = i

        zhi_index = 26 - (month_zhi_index + time_zhi_index)
        if zhi_index > 12:
            zhi_index -= 12
        jia_zi_index = LunarUtil.getJiaZiIndex(self.getMonth()) - (month_zhi_index - zhi_index)
        if jia_zi_index >= 60:
            jia_zi_index -= 60
        if jia_zi_index < 0:
            jia_zi_index += 60
        return LunarUtil.JIA_ZI[jia_zi_index]

    def getMingGongNaYin(self):
        """
        获取命宫纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getMingGong())

    def getShenGong(self):
        """
        获取身宫
        :return: 身宫
        """
        month_zhi_index = 0
        time_zhi_index = 0
        for i in range(0, len(EightChar.MONTH_ZHI)):
            zhi = EightChar.MONTH_ZHI[i]
            if self.__lunar.getMonthZhiExact() == zhi:
                month_zhi_index = i

            if self.__lunar.getTimeZhi() == zhi:
                time_zhi_index = i

        zhi_index = (2 + (month_zhi_index + time_zhi_index)) % 12
        jia_zi_index = LunarUtil.getJiaZiIndex(self.getMonth()) - (month_zhi_index - zhi_index)
        if jia_zi_index >= 60:
            jia_zi_index -= 60
        if jia_zi_index < 0:
            jia_zi_index += 60
        return LunarUtil.JIA_ZI[jia_zi_index]

    def getShenGongNaYin(self):
        """
        获取身宫纳音
        :return: 纳音
        """
        return LunarUtil.NAYIN.get(self.getShenGong())

    def getLunar(self):
        return self.__lunar

    def getYun(self, gender):
        """
        获取运
        :param gender: 性别：1男，0女
        :return: 运
        """
        from .eightchar import Yun
        return Yun(self, gender)

    def getYearXun(self):
        """
        获取年柱所在旬
        :return: 旬
        """
        return self.__lunar.getYearXunExact()

    def getYearXunKong(self):
        """
        获取年柱旬空(空亡)
        :return: 旬空(空亡)
        """
        return self.__lunar.getYearXunKongExact()

    def getMonthXun(self):
        """
        获取月柱所在旬
        :return: 旬
        """
        return self.__lunar.getMonthXunExact()

    def getMonthXunKong(self):
        """
        获取月柱旬空(空亡)
        :return: 旬空(空亡)
        """
        return self.__lunar.getMonthXunKongExact()

    def getDayXun(self):
        """
        获取日柱所在旬
        :return: 旬
        """
        return self.__lunar.getDayXunExact2() if 2 == self.__sect else self.__lunar.getDayXunExact()

    def getDayXunKong(self):
        """
        获取日柱旬空(空亡)
        :return: 旬空(空亡)
        """
        return self.__lunar.getDayXunKongExact2() if 2 == self.__sect else self.__lunar.getDayXunKongExact()

    def getTimeXun(self):
        """
        获取时柱所在旬
        :return: 旬
        """
        return self.__lunar.getTimeXun()

    def getTimeXunKong(self):
        """
        获取时柱旬空(空亡)
        :return: 旬空(空亡)
        """
        return self.__lunar.getTimeXunKong()



# -*- coding: utf-8 -*-
from datetime import datetime


class ExactDate:
    """
    精确日期
    """

    def __init__(self):
        pass

    @staticmethod
    def fromYmdHms(year, month, day, hour, minute, second):
        return datetime(year, month, day, hour, minute, second, 0)

    @staticmethod
    def fromYmd(year, month, day):
        return ExactDate.fromYmdHms(year, month, day, 0, 0, 0)

    @staticmethod
    def fromDate(date):
        return ExactDate.fromYmdHms(date.year, date.month, date.day, date.hour, date.minute, date.second)

    @staticmethod
    def getDaysBetween(ay, am, ad, by, bm, bd):
        from .util import SolarUtil
        if ay == by:
            n = SolarUtil.getDaysInYear(by, bm, bd) - SolarUtil.getDaysInYear(ay, am, ad)
        elif ay > by:
            days = SolarUtil.getDaysOfYear(by) - SolarUtil.getDaysInYear(by, bm, bd)
            for i in range(by + 1, ay):
                days += SolarUtil.getDaysOfYear(i)
            days += SolarUtil.getDaysInYear(ay, am, ad)
            n = -days
        else:
            days = SolarUtil.getDaysOfYear(ay) - SolarUtil.getDaysInYear(ay, am, ad)
            for i in range(ay + 1, by):
                days += SolarUtil.getDaysOfYear(i)
            days += SolarUtil.getDaysInYear(by, bm, bd)
            n = days
        return n

    @staticmethod
    def getDaysBetweenDate(date0, date1):
        return ExactDate.getDaysBetween(date0.year, date0.month, date0.day, date1.year, date1.month, date1.day)



# -*- coding: utf-8 -*-
from . import Lunar, TaoFestival
from .util import LunarUtil, TaoUtil


class Tao:
    """
    道历
    """

    BIRTH_YEAR = -2697

    def __init__(self, lunar):
        self.__lunar = lunar

    @staticmethod
    def fromLunar(lunar):
        return Tao(lunar)

    @staticmethod
    def fromYmdHms(year, month, day, hour, minute, second):
        return Tao.fromLunar(Lunar.fromYmdHms(year + Tao.BIRTH_YEAR, month, day, hour, minute, second))

    @staticmethod
    def fromYmd(year, month, day):
        return Tao.fromYmdHms(year, month, day, 0, 0, 0)

    def getLunar(self):
        return self.__lunar

    def getYear(self):
        return self.__lunar.getYear() - Tao.BIRTH_YEAR

    def getMonth(self):
        return self.__lunar.getMonth()

    def getDay(self):
        return self.__lunar.getDay()

    def getYearInChinese(self):
        y = str(self.getYear())
        s = ""
        for i in range(0, len(y)):
            s += LunarUtil.NUMBER[ord(y[i]) - 48]
        return s

    def getMonthInChinese(self):
        return self.__lunar.getMonthInChinese()

    def getDayInChinese(self):
        return self.__lunar.getDayInChinese()

    def getFestivals(self):
        festivals = []
        md = "%d-%d" % (self.getMonth(), self.getDay())
        if md in TaoUtil.FESTIVAL:
            fs = TaoUtil.FESTIVAL[md]
            for f in fs:
                festivals.append(f)
        jq = self.__lunar.getJieQi()
        if "冬至" == jq:
            festivals.append(TaoFestival("元始天尊圣诞"))
        elif "夏至" == jq:
            festivals.append(TaoFestival("灵宝天尊圣诞"))
        # 八节日
        if jq in TaoUtil.BA_JIE:
            festivals.append(TaoFestival(TaoUtil.BA_JIE[jq]))
        # 八会日
        gz = self.__lunar.getDayInGanZhi()
        if gz in TaoUtil.BA_HUI:
            festivals.append(TaoFestival(TaoUtil.BA_HUI[gz]))
        return festivals

    def __isDayIn(self, days):
        md = "%d-%d" % (self.getMonth(), self.getDay())
        for d in days:
            if md == d:
                return True
        return False

    def isDaySanHui(self):
        return self.__isDayIn(TaoUtil.SAN_HUI)

    def isDaySanYuan(self):
        return self.__isDayIn(TaoUtil.SAN_YUAN)

    def isDayBaJie(self):
        return self.__lunar.getJieQi() in TaoUtil.BA_JIE

    def isDayWuLa(self):
        return self.__isDayIn(TaoUtil.WU_LA)

    def isDayBaHui(self):
        return self.__lunar.getDayInGanZhi() in TaoUtil.BA_HUI

    def isDayMingWu(self):
        return "戊" == self.__lunar.getDayGan()

    def isDayAnWu(self):
        return self.__lunar.getDayZhi() == TaoUtil.AN_WU[abs(self.getMonth()) - 1]

    def isDayWu(self):
        return self.isDayMingWu() or self.isDayAnWu()

    def isDayTianShe(self):
        ret = False
        mz = self.__lunar.getMonthZhi()
        dgz = self.__lunar.getDayInGanZhi()
        if mz in "寅卯辰":
            if "戊寅" == dgz:
                ret = True
        elif mz in "巳午未":
            if "甲午" == dgz:
                ret = True
        elif mz in "申酉戌":
            if "戊申" == dgz:
                ret = True
        elif mz in "亥子丑":
            if "甲子" == dgz:
                ret = True
        return ret

    def __str__(self):
        return self.toString()

    def toString(self):
        return "%s年%s月%s" % (self.getYearInChinese(), self.getMonthInChinese(), self.getDayInChinese())

    def toFullString(self):
        return "道歷%s年，天运%s年，%s月，%s日。%s月%s日，%s時。" % (self.getYearInChinese(), self.__lunar.getYearInGanZhi(), self.__lunar.getMonthInGanZhi(), self.__lunar.getDayInGanZhi(), self.getMonthInChinese(), self.getDayInChinese(), self.__lunar.getTimeZhi())



# -*- coding: utf-8 -*-
from . import Solar, LunarYear, NineStar
from .util import LunarUtil


class LunarMonth:
    """
    农历月
    """

    def __init__(self, lunar_year, lunar_month, day_count, first_julian_day):
        self.__year = lunar_year
        self.__month = lunar_month
        self.__dayCount = day_count
        self.__firstJulianDay = first_julian_day

    @staticmethod
    def fromYm(lunar_year, lunar_month):
        from . import LunarYear
        return LunarYear.fromYear(lunar_year).getMonth(lunar_month)

    def getYear(self):
        return self.__year

    def getMonth(self):
        return self.__month

    def isLeap(self):
        return self.__month < 0

    def getDayCount(self):
        return self.__dayCount

    def getFirstJulianDay(self):
        return self.__firstJulianDay

    def getPositionTaiSui(self):
        m = abs(self.__month) % 4
        if 0 == m:
            p = "巽"
        elif 1 == m:
            p = "艮"
        elif 3 == m:
            p = "坤"
        else:
            p = LunarUtil.POSITION_GAN[Solar.fromJulianDay(self.getFirstJulianDay()).getLunar().getMonthGanIndex()]
        return p

    def getPositionTaiSuiDesc(self):
        return LunarUtil.POSITION_DESC[self.getPositionTaiSui()]

    def getNineStar(self):
        index = LunarYear.fromYear(self.__year).getZhiIndex() % 3
        m = abs(self.__month)
        month_zhi_index = (13 + m) % 12
        n = 27 - (index * 3)
        if month_zhi_index < LunarUtil.BASE_MONTH_ZHI_INDEX:
            n -= 3
        offset = (n - month_zhi_index) % 9
        return NineStar.fromIndex(offset)

    def toString(self):
        return "%d年%s%s月(%d天)" % (self.__year, ("闰" if self.isLeap() else ""), LunarUtil.MONTH[abs(self.__month)], self.__dayCount)

    def __str__(self):
        return self.toString()

    def next(self, n):
        """
        获取往后推几个月的阴历月，如果要往前推，则月数用负数
        :param n: 月数
        :return: 阴历月
        """
        if 0 == n:
            return LunarMonth.fromYm(self.__year, self.__month)
        elif n > 0:
            rest = n
            ny = self.__year
            iy = ny
            im = self.__month
            index = 0
            months = LunarYear.fromYear(ny).getMonths()
            while True:
                size = len(months)
                for i in range(0, size):
                    m = months[i]
                    if m.getYear() == iy and m.getMonth() == im:
                        index = i
                        break
                more = size - index - 1
                if rest < more:
                    break
                rest -= more
                last_month = months[size - 1]
                iy = last_month.getYear()
                im = last_month.getMonth()
                ny += 1
                months = LunarYear.fromYear(ny).getMonths()
            return months[index + rest]
        else:
            rest = -n
            ny = self.__year
            iy = ny
            im = self.__month
            index = 0
            months = LunarYear.fromYear(ny).getMonths()
            while True:
                size = len(months)
                for i in range(0, size):
                    m = months[i]
                    if m.getYear() == iy and m.getMonth() == im:
                        index = i
                        break
                if rest <= index:
                    break
                rest -= index
                first_month = months[0]
                iy = first_month.getYear()
                im = first_month.getMonth()
                ny -= 1
                months = LunarYear.fromYear(ny).getMonths()
            return months[index - rest]



# -*- coding: utf-8 -*-
class Holiday:
    """
    节假日
    """

    def __init__(self, day, name, work, target):
        """
        初始化
        :param day: 日期，YYYY-MM-DD格式
        :param name: 名称，如：国庆
        :param work: 是否调休，即是否要上班
        :param target: 关联的节日，YYYY-MM-DD格式
        """
        self.__day = Holiday.__ymd(day)
        self.__name = name
        self.__work = work
        self.__target = Holiday.__ymd(target)

    @staticmethod
    def __ymd(s):
        return s if "-" in s else (s[0:4] + "-" + s[4:6] + "-" + s[6:])

    def getDay(self):
        return self.__day

    def getName(self):
        return self.__name

    def isWork(self):
        return self.__work

    def getTarget(self):
        return self.__target

    def setDay(self, day):
        self.__day = Holiday.__ymd(day)

    def setName(self, name):
        self.__name = name

    def setWork(self, work):
        self.__work = work

    def setTarget(self, target):
        self.__target = Holiday.__ymd(target)

    def toString(self):
        return "%s %s%s %s" % (self.__day, self.__name, "调休" if self.__work else "", self.__target)

    def __str__(self):
        return self.toString()



# -*- coding: utf-8 -*-
from .HolidayUtil import HolidayUtil
from .ShouXingUtil import ShouXingUtil
from .SolarUtil import SolarUtil
from .LunarUtil import LunarUtil
from .FotoUtil import FotoUtil
from .TaoUtil import TaoUtil



# -*- coding: utf-8 -*-
from .ExactDate import ExactDate
from .JieQi import JieQi
from .NineStar import NineStar
from .EightChar import EightChar
from .ShuJiu import ShuJiu
from .Fu import Fu
from .Solar import Solar
from .SolarWeek import SolarWeek
from .SolarMonth import SolarMonth
from .SolarSeason import SolarSeason
from .SolarHalfYear import SolarHalfYear
from .SolarYear import SolarYear
from .Lunar import Lunar
from .LunarYear import LunarYear
from .LunarMonth import LunarMonth
from .LunarTime import LunarTime
from .Holiday import Holiday
from .FotoFestival import FotoFestival
from .Foto import Foto
from .TaoFestival import TaoFestival
from .Tao import Tao



# -*- coding: utf-8 -*-
from . import Lunar, LunarMonth
from .util import LunarUtil, FotoUtil


class Foto:
    """
    佛历
    """

    DEAD_YEAR = -543

    def __init__(self, lunar):
        self.__lunar = lunar

    @staticmethod
    def fromLunar(lunar):
        return Foto(lunar)

    @staticmethod
    def fromYmdHms(year, month, day, hour, minute, second):
        return Foto.fromLunar(Lunar.fromYmdHms(year + Foto.DEAD_YEAR - 1, month, day, hour, minute, second))

    @staticmethod
    def fromYmd(year, month, day):
        return Foto.fromYmdHms(year, month, day, 0, 0, 0)

    def getLunar(self):
        return self.__lunar

    def getYear(self):
        sy = self.__lunar.getSolar().getYear()
        y = sy - Foto.DEAD_YEAR
        if sy == self.__lunar.getYear():
            y += 1
        return y

    def getMonth(self):
        return self.__lunar.getMonth()

    def getDay(self):
        return self.__lunar.getDay()

    def getYearInChinese(self):
        y = str(self.getYear())
        s = ""
        for i in range(0, len(y)):
            s += LunarUtil.NUMBER[ord(y[i]) - 48]
        return s

    def getMonthInChinese(self):
        return self.__lunar.getMonthInChinese()

    def getDayInChinese(self):
        return self.__lunar.getDayInChinese()

    def getFestivals(self):
        festivals = []
        md = "%d-%d" % (self.getMonth(), self.getDay())
        if md in FotoUtil.FESTIVAL:
            fs = FotoUtil.FESTIVAL[md]
            for f in fs:
                festivals.append(f)
        return festivals

    def isMonthZhai(self):
        m = self.getMonth()
        return 1 == m or 5 == m or 9 == m

    def isDayYangGong(self):
        for f in self.getFestivals():
            if "杨公忌" == f.getName():
                return True
        return False

    def isDayZhaiShuoWang(self):
        d = self.getDay()
        return 1 == d or 15 == d

    def isDayZhaiSix(self):
        d = self.getDay()
        if 8 == d or 14 == d or 15 == d or 23 == d or 29 == d or 30 == d:
            return True
        elif 28 == d:
            m = LunarMonth.fromYm(self.__lunar.getYear(), self.getMonth())
            return m is not None and 30 != m.getDayCount()
        return False

    def isDayZhaiTen(self):
        d = self.getDay()
        return 1 == d or 8 == d or 14 == d or 15 == d or 18 == d or 23 == d or 24 == d or 28 == d or 29 == d or 30 == d

    def isDayZhaiGuanYin(self):
        k = "%d-%d" % (self.getMonth(), self.getDay())
        for d in FotoUtil.DAY_ZHAI_GUAN_YIN:
            if k == d:
                return True
        return False

    def getXiu(self):
        return FotoUtil.getXiu(self.getMonth(), self.getDay())

    def getXiuLuck(self):
        return LunarUtil.XIU_LUCK[self.getXiu()]

    def getXiuSong(self):
        return LunarUtil.XIU_SONG[self.getXiu()]

    def getZheng(self):
        return LunarUtil.ZHENG[self.getXiu()]

    def getAnimal(self):
        return LunarUtil.ANIMAL[self.getXiu()]

    def getGong(self):
        return LunarUtil.GONG[self.getXiu()]

    def getShou(self):
        return LunarUtil.SHOU[self.getGong()]

    def __str__(self):
        return self.toString()

    def toString(self):
        return "%s年%s月%s" % (self.getYearInChinese(), self.getMonthInChinese(), self.getDayInChinese())

    def toFullString(self):
        s = self.toString()
        for f in self.getFestivals():
            s += " (%s)" % f
        return s



# -*- coding: utf-8 -*-
from ..TaoFestival import TaoFestival


class TaoUtil:
    """
    道历工具
    """

    # 三会日
    SAN_HUI = ("1-7", "7-7", "10-15")

    # 三元日
    SAN_YUAN = ("1-15", "7-15", "10-15")

    # 五腊日
    WU_LA = ("1-1", "5-5", "7-7", "10-1", "12-8")

    # 暗戊
    AN_WU = ("未", "戌", "辰", "寅", "午", "子", "酉", "申", "巳", "亥", "卯", "丑")

    # 八会日
    BA_HUI = {
        "丙午": "天会",
        "壬午": "地会",
        "壬子": "人会",
        "庚午": "日会",
        "庚申": "月会",
        "辛酉": "星辰会",
        "甲辰": "五行会",
        "甲戌": "四时会"
    }

    # 八节日
    BA_JIE = {
        "立春": "东北方度仙上圣天尊同梵炁始青天君下降",
        "春分": "东方玉宝星上天尊同青帝九炁天君下降",
        "立夏": "东南方好生度命天尊同梵炁始丹天君下降",
        "夏至": "南方玄真万福天尊同赤帝三炁天君下降",
        "立秋": "西南方太灵虚皇天尊同梵炁始素天君下降",
        "秋分": "西方太妙至极天尊同白帝七炁天君下降",
        "立冬": "西北方无量太华天尊同梵炁始玄天君下降",
        "冬至": "北方玄上玉宸天尊同黑帝五炁天君下降"
    }

    # 日期对应的节日
    FESTIVAL = {
        "1-1": [TaoFestival("天腊之辰", "天腊，此日五帝会于东方九炁青天")],
        "1-3": [TaoFestival("郝真人圣诞"), TaoFestival("孙真人圣诞")],
        "1-5": [TaoFestival("孙祖清静元君诞")],
        "1-7": [TaoFestival("举迁赏会", "此日上元赐福，天官同地水二官考校罪福")],
        "1-9": [TaoFestival("玉皇上帝圣诞")],
        "1-13": [TaoFestival("关圣帝君飞升")],
        "1-15": [TaoFestival("上元天官圣诞"), TaoFestival("老祖天师圣诞")],
        "1-19": [TaoFestival("长春邱真人(邱处机)圣诞")],
        "1-28": [TaoFestival("许真君(许逊天师)圣诞")],
        "2-1": [TaoFestival("勾陈天皇大帝圣诞"), TaoFestival("长春刘真人(刘渊然)圣诞")],
        "2-2": [TaoFestival("土地正神诞"), TaoFestival("姜太公圣诞")],
        "2-3": [TaoFestival("文昌梓潼帝君圣诞")],
        "2-6": [TaoFestival("东华帝君圣诞")],
        "2-13": [TaoFestival("度人无量葛真君圣诞")],
        "2-15": [TaoFestival("太清道德天尊(太上老君)圣诞")],
        "2-19": [TaoFestival("慈航真人圣诞")],
        "3-1": [TaoFestival("谭祖(谭处端)长真真人圣诞")],
        "3-3": [TaoFestival("玄天上帝圣诞")],
        "3-6": [TaoFestival("眼光娘娘圣诞")],
        "3-15": [TaoFestival("天师张大真人圣诞"), TaoFestival("财神赵公元帅圣诞")],
        "3-16": [TaoFestival("三茅真君得道之辰"), TaoFestival("中岳大帝圣诞")],
        "3-18": [TaoFestival("王祖(王处一)玉阳真人圣诞"), TaoFestival("后土娘娘圣诞")],
        "3-19": [TaoFestival("太阳星君圣诞")],
        "3-20": [TaoFestival("子孙娘娘圣诞")],
        "3-23": [TaoFestival("天后妈祖圣诞")],
        "3-26": [TaoFestival("鬼谷先师诞")],
        "3-28": [TaoFestival("东岳大帝圣诞")],
        "4-1": [TaoFestival("长生谭真君成道之辰")],
        "4-10": [TaoFestival("何仙姑圣诞")],
        "4-14": [TaoFestival("吕祖纯阳祖师圣诞")],
        "4-15": [TaoFestival("钟离祖师圣诞")],
        "4-18": [TaoFestival("北极紫微大帝圣诞"), TaoFestival("泰山圣母碧霞元君诞"), TaoFestival("华佗神医先师诞")],
        "4-20": [TaoFestival("眼光圣母娘娘诞")],
        "4-28": [TaoFestival("神农先帝诞")],
        "5-1": [TaoFestival("南极长生大帝圣诞")],
        "5-5": [TaoFestival("地腊之辰", "地腊，此日五帝会于南方三炁丹天"), TaoFestival("南方雷祖圣诞"), TaoFestival("地祗温元帅圣诞"), TaoFestival("雷霆邓天君圣诞")],
        "5-11": [TaoFestival("城隍爷圣诞")],
        "5-13": [TaoFestival("关圣帝君降神"), TaoFestival("关平太子圣诞")],
        "5-18": [TaoFestival("张天师圣诞")],
        "5-20": [TaoFestival("马祖丹阳真人圣诞")],
        "5-29": [TaoFestival("紫青白祖师圣诞")],
        "6-1": [TaoFestival("南斗星君下降")],
        "6-2": [TaoFestival("南斗星君下降")],
        "6-3": [TaoFestival("南斗星君下降")],
        "6-4": [TaoFestival("南斗星君下降")],
        "6-5": [TaoFestival("南斗星君下降")],
        "6-6": [TaoFestival("南斗星君下降")],
        "6-10": [TaoFestival("刘海蟾祖师圣诞")],
        "6-15": [TaoFestival("灵官王天君圣诞")],
        "6-19": [TaoFestival("慈航(观音)成道日")],
        "6-23": [TaoFestival("火神圣诞")],
        "6-24": [TaoFestival("南极大帝中方雷祖圣诞"), TaoFestival("关圣帝君圣诞")],
        "6-26": [TaoFestival("二郎真君圣诞")],
        "7-7": [TaoFestival("道德腊之辰", "道德腊，此日五帝会于西方七炁素天"), TaoFestival("庆生中会", "此日中元赦罪，地官同天水二官考校罪福")],
        "7-12": [TaoFestival("西方雷祖圣诞")],
        "7-15": [TaoFestival("中元地官大帝圣诞")],
        "7-18": [TaoFestival("王母娘娘圣诞")],
        "7-20": [TaoFestival("刘祖(刘处玄)长生真人圣诞")],
        "7-22": [TaoFestival("财帛星君文财神增福相公李诡祖圣诞")],
        "7-26": [TaoFestival("张三丰祖师圣诞")],
        "8-1": [TaoFestival("许真君飞升日")],
        "8-3": [TaoFestival("九天司命灶君诞")],
        "8-5": [TaoFestival("北方雷祖圣诞")],
        "8-10": [TaoFestival("北岳大帝诞辰")],
        "8-15": [TaoFestival("太阴星君诞")],
        "9-1": [TaoFestival("北斗九皇降世之辰")],
        "9-2": [TaoFestival("北斗九皇降世之辰")],
        "9-3": [TaoFestival("北斗九皇降世之辰")],
        "9-4": [TaoFestival("北斗九皇降世之辰")],
        "9-5": [TaoFestival("北斗九皇降世之辰")],
        "9-6": [TaoFestival("北斗九皇降世之辰")],
        "9-7": [TaoFestival("北斗九皇降世之辰")],
        "9-8": [TaoFestival("北斗九皇降世之辰")],
        "9-9": [TaoFestival("北斗九皇降世之辰"), TaoFestival("斗姥元君圣诞"), TaoFestival("重阳帝君圣诞"), TaoFestival("玄天上帝飞升"), TaoFestival("酆都大帝圣诞")],
        "9-22": [TaoFestival("增福财神诞")],
        "9-23": [TaoFestival("萨翁真君圣诞")],
        "9-28": [TaoFestival("五显灵官马元帅圣诞")],
        "10-1": [TaoFestival("民岁腊之辰", "民岁腊，此日五帝会于北方五炁黑天"), TaoFestival("东皇大帝圣诞")],
        "10-3": [TaoFestival("三茅应化真君圣诞")],
        "10-6": [TaoFestival("天曹诸司五岳五帝圣诞")],
        "10-15": [TaoFestival("下元水官大帝圣诞"), TaoFestival("建生大会", "此日下元解厄，水官同天地二官考校罪福")],
        "10-18": [TaoFestival("地母娘娘圣诞")],
        "10-19": [TaoFestival("长春邱真君飞升")],
        "10-20": [TaoFestival("虚靖天师(即三十代天师弘悟张真人)诞")],
        "11-6": [TaoFestival("西岳大帝圣诞")],
        "11-9": [TaoFestival("湘子韩祖圣诞")],
        "11-11": [TaoFestival("太乙救苦天尊圣诞")],
        "11-26": [TaoFestival("北方五道圣诞")],
        "12-8": [TaoFestival("王侯腊之辰", "王侯腊，此日五帝会于上方玄都玉京")],
        "12-16": [TaoFestival("南岳大帝圣诞"), TaoFestival("福德正神诞")],
        "12-20": [TaoFestival("鲁班先师圣诞")],
        "12-21": [TaoFestival("天猷上帝圣诞")],
        "12-22": [TaoFestival("重阳祖师圣诞")],
        "12-23": [TaoFestival("祭灶王", "最适宜谢旧年太岁，开启拜新年太岁")],
        "12-25": [TaoFestival("玉帝巡天"), TaoFestival("天神下降")],
        "12-29": [TaoFestival("清静孙真君(孙不二)成道")]
    }

    def __init__(self):
        pass



# -*- coding: utf-8 -*-
from math import ceil

from .. import ExactDate


class SolarUtil:
    """
    阳历工具
    """

    # 星期
    WEEK = ("日", "一", "二", "三", "四", "五", "六")

    # 每月天数
    DAYS_OF_MONTH = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)

    # 星座
    XING_ZUO = ("白羊", "金牛", "双子", "巨蟹", "狮子", "处女", "天秤", "天蝎", "射手", "摩羯", "水瓶", "双鱼")

    # 日期对应的节日
    FESTIVAL = {
        "1-1": "元旦节",
        "2-14": "情人节",
        "3-8": "妇女节",
        "3-12": "植树节",
        "3-15": "消费者权益日",
        "4-1": "愚人节",
        "5-1": "劳动节",
        "5-4": "青年节",
        "6-1": "儿童节",
        "7-1": "建党节",
        "8-1": "建军节",
        "9-10": "教师节",
        "10-1": "国庆节",
        "12-24": "平安夜",
        "12-25": "圣诞节"
    }

    # 几月第几个星期几对应的节日
    WEEK_FESTIVAL = {
        "5-2-0": "母亲节",
        "6-3-0": "父亲节",
        "11-4-4": "感恩节"
    }

    # 日期对应的非正式节日
    OTHER_FESTIVAL = {
        "1-8": ["周恩来逝世纪念日"],
        "1-10": ["中国人民警察节", "中国公安110宣传日"],
        "1-21": ["列宁逝世纪念日"],
        "1-26": ["国际海关日"],
        "2-2": ["世界湿地日"],
        "2-4": ["世界抗癌日"],
        "2-7": ["京汉铁路罢工纪念"],
        "2-10": ["国际气象节"],
        "2-19": ["邓小平逝世纪念日"],
        "2-21": ["国际母语日"],
        "2-24": ["第三世界青年日"],
        "3-1": ["国际海豹日"],
        "3-3": ["全国爱耳日"],
        "3-5": ["周恩来诞辰纪念日", "中国青年志愿者服务日"],
        "3-6": ["世界青光眼日"],
        "3-12": ["孙中山逝世纪念日"],
        "3-14": ["马克思逝世纪念日"],
        "3-17": ["国际航海日"],
        "3-18": ["全国科技人才活动日"],
        "3-21": ["世界森林日", "世界睡眠日"],
        "3-22": ["世界水日"],
        "3-23": ["世界气象日"],
        "3-24": ["世界防治结核病日"],
        "4-2": ["国际儿童图书日"],
        "4-7": ["世界卫生日"],
        "4-22": ["列宁诞辰纪念日"],
        "4-23": ["世界图书和版权日"],
        "4-26": ["世界知识产权日"],
        "5-3": ["世界新闻自由日"],
        "5-5": ["马克思诞辰纪念日"],
        "5-8": ["世界红十字日"],
        "5-11": ["世界肥胖日"],
        "5-27": ["上海解放日"],
        "5-31": ["世界无烟日"],
        "6-5": ["世界环境日"],
        "6-6": ["全国爱眼日"],
        "6-8": ["世界海洋日"],
        "6-11": ["中国人口日"],
        "6-14": ["世界献血日"],
        "7-1": ["香港回归纪念日"],
        "7-7": ["中国人民抗日战争纪念日"],
        "7-11": ["世界人口日"],
        "8-5": ["恩格斯逝世纪念日"],
        "8-6": ["国际电影节"],
        "8-12": ["国际青年日"],
        "8-22": ["邓小平诞辰纪念日"],
        "9-3": ["中国抗日战争胜利纪念日"],
        "9-8": ["世界扫盲日"],
        "9-9": ["毛泽东逝世纪念日"],
        "9-14": ["世界清洁地球日"],
        "9-18": ["九一八事变纪念日"],
        "9-20": ["全国爱牙日"],
        "9-21": ["国际和平日"],
        "9-27": ["世界旅游日"],
        "10-4": ["世界动物日"],
        "10-10": ["辛亥革命纪念日"],
        "10-13": ["中国少年先锋队诞辰日"],
        "10-25": ["抗美援朝纪念日"],
        "11-12": ["孙中山诞辰纪念日"],
        "11-17": ["国际大学生节"],
        "11-28": ["恩格斯诞辰纪念日"],
        "12-1": ["世界艾滋病日"],
        "12-12": ["西安事变纪念日"],
        "12-13": ["国家公祭日"],
        "12-26": ["毛泽东诞辰纪念日"]
    }

    def __init__(self):
        pass

    @staticmethod
    def isLeapYear(year):
        """
        是否闰年
        :param year: 年
        :return: True/False 闰年/非闰年
        """
        return (year % 4 == 0 and year % 100 != 0) or (year % 400 == 0)

    @staticmethod
    def getDaysOfYear(year):
        d = 365
        if SolarUtil.isLeapYear(year):
            d = 366
        return d

    @staticmethod
    def getDaysOfMonth(year, month):
        """
        获取某年某月有多少天
        :param year: 年
        :param month: 月
        :return: 天数
        """
        if 1582 == year and 10 == month:
            return 21
        d = SolarUtil.DAYS_OF_MONTH[month - 1]
        # 公历闰年2月多一天
        if month == 2 and SolarUtil.isLeapYear(year):
            d += 1
        return d

    @staticmethod
    def getDaysInYear(year, month, day):
        days = 0
        for i in range(1, month):
            days += SolarUtil.getDaysOfMonth(year, i)
        days += day
        if 1582 == year and 10 == month and day >= 15:
            days -= 10
        return days

    @staticmethod
    def getWeeksOfMonth(year, month, start):
        """
        获取某年某月有多少周
        :param year: 年
        :param month: 月
        :param start: 星期几作为一周的开始，1234560分别代表星期一至星期天
        :return: 天数
        """
        days = SolarUtil.getDaysOfMonth(year, month)
        week = ExactDate.fromYmd(year, month, 1).isoweekday()
        if week == 7:
            week = 0
        return int(ceil((days + week - start) * 1.0 / len(SolarUtil.WEEK)))
